/*******************************************************************************
   BH1900NUX.h

   Copyright (c) 2022 ROHM Co.,Ltd.

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*******************************************************************************/
#ifndef BH1900NUX_H__
#define BH1900NUX_H__

#ifndef float32
#define float32                            float
#endif

// Error Number
#define BH1900NUX_OK                       (0)
#define BH1900NUX_COMM_ERROR               (-1)
#define BH1900NUX_WAI_ERROR                (-2)
#define BH1900NUX_TEMP_ERROR               (-3)
// Slave Address
#define BH1900NUX_SLAVE_ADDRESS_BASE       (0x48) //The slave address is selectable from 0x48 to 0x4F by A0, A1, A2 pins.
#define BH1900NUX_SLAVE_ADDRESS_DEFAULT    (1)
#define BH1900NUX_SLAVE_ADDRESS_MAXNUM     (8)
// Register Address
#define BH1900NUX_TEMPERATURE              (0x00)
#define BH1900NUX_CONFIG                   (0x01)
#define BH1900NUX_TLOW                     (0x02)
#define BH1900NUX_THIGH                    (0x03)
#define BH1900NUX_SW_RESET                 (0x04)
// Register Parameter
#define BH1900NUX_CONFIG_OS_CONTINUOUS     (0UL << 15)
#define BH1900NUX_CONFIG_ALERT_OFF         (0UL << 14)
#define BH1900NUX_CONFIG_FAULTQUEUE_SINGLE (0UL << 11)
#define BH1900NUX_CONFIG_POL_LOW           (0UL << 10)
#define BH1900NUX_CONFIG_SD_ACTIVE         (0UL << 8)
#define BH1900NUX_CONFIG_SD_SLEEP          (1UL << 8)
#define BH1900NUX_CONFIG_WT_X1             (2UL << 0)
#define BH1900NUX_SW_RESET_VAL             (1UL << 0)
// Register Value(write)
#define BH1900NUX_CONFIG_VAL               (BH1900NUX_CONFIG_OS_CONTINUOUS | BH1900NUX_CONFIG_ALERT_OFF | BH1900NUX_CONFIG_FAULTQUEUE_SINGLE | BH1900NUX_CONFIG_POL_LOW | BH1900NUX_CONFIG_SD_SLEEP | BH1900NUX_CONFIG_WT_X1)
// Register Value(read)
#define BH1900NUX_SW_RESET_DEFAULT_VAL     (0xFF) //Use as Who am I
//Meas Time
#define BH1900NUX_CONFIG_WT_X16_MAX        (6376) //ms  35 + (186240*16 )/470
#define BH1900NUX_CONFIG_WT_X4_MAX         (1621) //ms  35 + (186240*4  )/470
#define BH1900NUX_CONFIG_WT_X1_MAX         (432)  //ms  35 + (186240*1  )/470
#define BH1900NUX_CONFIG_WT_X0_5_MAX       (234)  //ms  35 + (186240*0.5)/470
#define BH1900NUX_CONFIG_WT_MODE_NUM       (4)
//Temperature Value
#define BH1900NUX_OPERATING_TEMP_MAX       (95.0000)
#define BH1900NUX_OPERATING_TEMP_MIN       (-30.0000)
#define BH1900NUX_CONVERT_TEMP             (256)  // 1 / (0.0625 * (1 >> 4))
// Mask
#define BH1900NUX_CONFIG_MASK              (0xFEFF)
#define BH1900NUX_CONFIG_WT_MASK           (0x0003)
// Version
#define BH1900NUX_DRIVER_VERSION           (1.1F)
// Others
#define BH1900NUX_TEMPERATURE_DATA_SIZE    (2)
#define BH1900NUX_WRITE_DATA_SIZE          (2)

class BH1900NUX
{
    public:
        BH1900NUX(uint8_t addr_pin);
        ~BH1900NUX();
        int32_t init(void);
        int32_t start(void);
        int32_t stop(void);
        int32_t get_val(uint8_t *raw);
        int32_t get_val(float32 *temp);
        float32 get_version(void);

    private:
        uint8_t Address;

        int32_t  init_check_val(void);
        int32_t  init_setting_ic(void);
        uint32_t setting_wait(uint16_t config);
        int32_t  write(uint8_t reg, uint8_t *data, int32_t size);
        int32_t  read(uint8_t reg, uint8_t *data, int32_t size);
};

#endif // BH1900NUX_H__