/*******************************************************************************
   BH1900NUX.ino

   Copyright (c) 2022 ROHM Co.,Ltd.

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*******************************************************************************/
#include <Arduino.h>
#include <HardwareSerial.h>
#include <Wire.h>
#include "BH1900NUX.h"

#define SYSTEM_BAUDRATE (115200)
#define TEMP_DISP       (4)
#define ERROR_WAIT      (1000)
#define ERROR_POINT     (30)

uint16_t meas_time;

BH1900NUX bh1900(BH1900NUX_SLAVE_ADDRESS_DEFAULT);
void error_func(int32_t result);

void setup()
{
    int32_t result;
    float32 version;

    (void)Serial.begin(SYSTEM_BAUDRATE);
    while(!Serial){
    }

    (void)Serial.println("BH1900NUX Sample Code Version : 1.1");

    version = bh1900.get_version();
    (void)Serial.print("BH1900NUX Driver Version : ");
    (void)Serial.println(version, 1);

    Wire.begin();

    result = bh1900.init();
    if (result > BH1900NUX_OK) {
        meas_time = result;
        (void)bh1900.start();
    } else {
        error_func(result);
    }

    delay(meas_time);

    return;
}

void loop()
{
    int32_t result;
    float32 temp;

    result = bh1900.get_val(&temp);
    if (result == BH1900NUX_COMM_ERROR) {
        error_func(result);
    } else {
        (void)Serial.print("TEMP(Celsius)= ");
        (void)Serial.println(temp, TEMP_DISP);
        if (result == BH1900NUX_OK) {
            // non process
        } else {
            (void)Serial.println("Out of the Operating Temperature Range.");
        }
    }

    delay(meas_time);

    return;
}

void error_func(int32_t result)
{
    uint8_t cnt = 0;

    switch (result) {
        case BH1900NUX_COMM_ERROR :
            (void)Serial.println("Communication Error.");
            break;

        case BH1900NUX_WAI_ERROR :
            (void)Serial.println("Parameter Error.");
            break;

        default :
            (void)Serial.println("Unknown Error.");
            break;
    }

    (void)Serial.println("Check System and Driver Parameter.");
    while (1) {
        (void)Serial.print(".");
        if (cnt < ERROR_POINT) {
            cnt++;
        } else {
            cnt = 0;
            (void)Serial.println();
        }
        delay(ERROR_WAIT);
    }

    return;
}